//Include C++
#include <string>

//Include Boost
#include <boost/interprocess/managed_mapped_file.hpp>
#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/interprocess/mem_algo/rbtree_best_fit.hpp>
#include <boost/interprocess/mem_algo/simple_seq_fit.hpp>
#include <boost/interprocess/sync/mutex_family.hpp>
#include <boost/interprocess/indexes/iset_index.hpp>

//Namespace
namespace bi = boost::interprocess;

//Typedefs
typedef bi::basic_managed_mapped_file   <char, bi::simple_seq_fit<bi::mutex_family>,
                                         bi::iset_index> simple_managed_mapped_file;
typedef bi::managed_mapped_file managed_mapped_file;
typedef bi::managed_shared_memory::segment_manager segment_manager;

struct Message
{
  Message()
  { std::cout << "Message created" << std::endl; }

  ~Message()
  { std::cout << "Message destroyed" << std::endl; }

  size_t m_value;
};


int main ()
{
    const char * address = "test.map";
    size_t size = 10 * 1024 * 1024;
    std::remove(address);
    managed_mapped_file mapfile(bi::create_only, address, size);
    Message * message = (mapfile.construct<Message>(bi::anonymous_instance))();
    mapfile.destroy_ptr<Message>(message);

    return EXIT_SUCCESS;
}
